package net.ihe.gazelle.lib.hl7templatespackager.adapters;

import gnu.trove.map.hash.THashMap;
import net.ihe.gazelle.lib.hl7templatespackager.application.ValidatorGenerator;
import net.ihe.gazelle.lib.hl7templatespackager.application.Workspace;
import net.ihe.gazelle.lib.hl7templatespackager.application.exception.ValidatorGenerationException;
import net.ihe.gazelle.lib.hl7templatespackager.application.GOCLogger;
import net.ihe.gazelle.lib.hl7templatespackager.application.ValidatorConfiguration;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.io.IOException;
import java.util.Map;

/**
 * This class launches the execution of plugins used to generate code, documentation, etc
 * @author wbars
 */
public class ValidatorGeneratorPlugin implements ValidatorGenerator {

    private static Logger log = LoggerFactory.getLogger(ValidatorGeneratorPlugin.class);
    private static final String CD_COMMAND = "cd \"";
    private static final String VALIDATOR_JAR_SUFFIX = "-validator-jar";

    private String workspacesRoot;
    private String mvnExecutionPath;
    private GOCLogger gocLogger;
    private ProcessExecutor processExecutor;
    private ValidatorRightUpdater validatorRightUpdater;

    /**
     * Base constructor for the class
     */
    private ValidatorGeneratorPlugin(){
    }

    /**
     * Creates an instance of the class with a defined root path to create workspaces.
     * @param workspacesRoot : root path where workspaces will be created.
     */
    public ValidatorGeneratorPlugin(String workspacesRoot, String mvnExecutionPath, GOCLogger gocLogger, ProcessExecutor processExecutor,
                                    ValidatorRightUpdater validatorRightUpdater){
        setWorkspacesRoot(workspacesRoot);
        setMvnExecutionPath(mvnExecutionPath);
        this.gocLogger = gocLogger;
        this.processExecutor = processExecutor;
        this.validatorRightUpdater = validatorRightUpdater;
    }

    /**
     * Getter for the workspaceRoot property.
     */
    private String getWorkspacesRoot() {
        if (this.workspacesRoot != null){
            return this.workspacesRoot;
        }
        return "";
    }

    /**
     * Setter for the workspaceRoot property
     * @param workspacesRoot : the path we want to use as root.
     */
    private void setWorkspacesRoot(String workspacesRoot) {
        if (workspacesRoot != null){
            this.workspacesRoot = workspacesRoot;
        } else {
            this.workspacesRoot = "";
        }
    }

    /**
     * Getter for the workspaceRoot property.
     */
    private String getMvnExecutionPath() {
        if (this.mvnExecutionPath != null){
            return this.mvnExecutionPath;
        }
        return "mvn";
    }

    /**
     * Setter for the workspaceRoot property
     * @param mvnExecutionPath : the path we want to use as root.
     */
    private void setMvnExecutionPath(String mvnExecutionPath) {
        if (mvnExecutionPath != null){
            this.mvnExecutionPath = mvnExecutionPath;
        } else {
            this.mvnExecutionPath = "mvn";
        }
    }

    /**
     * Generate the java code for the validator by executing the maven goal mbgen:genvalidator
     * @param workspace : workspace where the java code will be generated.
     * @param validatorConfiguration : ValidatorConfiguration used for the generation. Contains info on the validator to be generated.
     * @throws ValidatorGenerationException when something went wrong on this step of the validator generation.
     */
    public void generateProjectValidator(Workspace workspace, ValidatorConfiguration validatorConfiguration) throws ValidatorGenerationException {
        log.warn("Generating validator by executing the maven goal 'exec:exec@generateValidator', all complete commands output logs could be found in cmdOutput.log");
        log.warn("Generated by ocl2java-1.1.4.jar library");
        //composite goal with mbgen:genvalidator & exec:exec@UpdateImports
        Map<String, String> params = new THashMap<>();
        String cdaProps = System.getProperty("HL7TEMP_CDACONFFOLDERNAME", "cdabasic");
        params.put("cdaProps",cdaProps);
        params.put("projectName",validatorConfiguration.getProjectNaming().getProjectName());
        params.put("mvnPath",getMvnExecutionPath());
        executeGoalWithParams(getProjectPath(workspace,validatorConfiguration),"exec:exec@generateValidator",params);
    }

    /**
     * Generate the CDA tree associated to the validator by executing the maven goal mbgen:gencdatree
     * @param workspace : workspace where the CDA tree will be generated.
     * @param validatorConfiguration : holds the name of the project that will be used to generate the CDA tree.
     * @throws ValidatorGenerationException when something went wrong on this step of the validator generation.
     */
    public void generateProjectCDATemplatesTree( Workspace workspace, ValidatorConfiguration validatorConfiguration) throws ValidatorGenerationException {
        log.warn("Generating CDA Templates Tree by executing the maven goal 'mbgen:gencdatree', all complete commands output logs could be found in cmdOutput.log");
        log.warn("Generated by ocl2java-1.1.4.jar library");
        executeGoal(getProjectPath(workspace, validatorConfiguration), "mbgen:gencdatree");

    }

    /**
     * Generate the documentation for the validator by executing the maven goal mbgen:genxmldoc
     * @param workspace : workspace where the documentation will be generated.
     * @param validatorConfiguration : holds the name of the project that will be used to generate the documentation.
     * @throws ValidatorGenerationException when something went wrong on this step of the validator generation.
     */
    public void extractDocumentation(Workspace workspace, ValidatorConfiguration validatorConfiguration) throws ValidatorGenerationException {
        log.warn("Generating Documentation by executing the maven goal 'mbgen:genxmldoc', all complete commands output logs could be found in cmdOutput.log");
        log.warn("Generated by ocl2java-1.1.4.jar library");
        executeGoal(getProjectPath(workspace, validatorConfiguration), "mbgen:genxmldoc");
    }

    /**
     * Packages the generated validator using maven package goal.
     * @param workspace : workspace where the project to be compiled is located.
     * @param validatorConfiguration : holds the name of the project that will be packaged.
     * @throws ValidatorGenerationException when something went wrong on this step of the validator generation.
     */
    public void assembleProjectValidator(Workspace workspace, ValidatorConfiguration validatorConfiguration) throws ValidatorGenerationException {
        log.warn("Executing 'mvn package' on  generated validator, all complete commands output logs could be found in cmdOutput.log");
        executeGoal(getProjectPath(workspace, validatorConfiguration), "package");
        try {
            gocLogger.logSeparator("Changing rights","chmod777 on appassembler/bin/validate.sh");
            validatorRightUpdater.updateValidatorRights(getProjectPath(workspace, validatorConfiguration));
        } catch (IOException e){
            throw new ValidatorGenerationException("Cannot change rights on Validator execution script !", e);
        }
    }

    public void updateUtilityScripts(Workspace workspace, ValidatorConfiguration validatorConfiguration) throws ValidatorGenerationException {
        log.warn("Executing 'chmod +x' on utility scripts before generating validator");
        try {
            validatorRightUpdater.updateUtilityScripts(getProjectPath(workspace,validatorConfiguration));

        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    private void executeGoal(String workspacePath, String goal) throws ValidatorGenerationException {
        try {
            executeMavenPluginTarget(workspacePath, getMvnExecutionPath(), goal);
        } catch (IOException e){
            throw  new ValidatorGenerationException("Cannot execute generation goal : " + goal, e);
        }
    }

    private void executeGoalWithParams(String workspacePath, String goal, Map<String, String> params) throws ValidatorGenerationException {
        try{
            executeMavenPluginWithParams(workspacePath,getMvnExecutionPath(),goal,params);
        } catch (IOException e) {
            throw new ValidatorGenerationException("Cannot execute generation goal : " + goal, e);
        }
    }

    /**
     * Return the project path based on worskpace path and ProjectNaming. This path is used to perform maven goals.
     * @param workspace : workspace where the project can be found.
     * @param validatorConfiguration : holds the name of the project.
     */
    private String getProjectPath(Workspace workspace, ValidatorConfiguration validatorConfiguration){
        return new File(getWorkspacesRoot() + workspace.getName()).getAbsolutePath() + File.separator + validatorConfiguration.getProjectNaming().getProjectName() + VALIDATOR_JAR_SUFFIX;
    }

    /**
     * Launches the execution of a maven goal passed as parameter on the path specified for the project.
     * @param projectFolder : path to the maven project on which the goal will be executed.
     * @param mavnExecPath : maven execution path (eg: mvn).
     * @param mavenGoal : maven goal to be executed (eg: package).
     * @throws IOException when there is an error running the command to execute the Maven goal.
     */
    private void executeMavenPluginTarget(String projectFolder, String mavnExecPath, String mavenGoal) throws IOException{
        String javaHomeCMD = "";
        if(System.getProperty("JAVA_EXEC") != null && !System.getProperty("JAVA_EXEC").isEmpty()){
            javaHomeCMD = " env $JAVA_HOME=\""+System.getProperty("JAVA_EXEC")+"\" ";
        }
        String execGen = CD_COMMAND + projectFolder + "\"" + System.lineSeparator() + javaHomeCMD + mavnExecPath + " " + mavenGoal + "\n";
        log.info("Executed mvn target : {}", execGen);
        processExecutor.executeProcess(execGen);
    }

    private void executeMavenPluginWithParams(String projectFolder, String mavnExecPath, String mavenGoal, Map<String,String> params) throws IOException{
        StringBuilder execGen = new StringBuilder(CD_COMMAND + projectFolder + "\"" + System.lineSeparator() + mavnExecPath + " " + mavenGoal);
        for(Map.Entry<String,String> param:params.entrySet()){
            execGen.append(" -D").append(param.getKey()).append("=\"").append(param.getValue()).append("\"");
        }
        execGen.append("\n");
        log.info("Executed mvn target : {}", execGen.toString());
        processExecutor.executeProcess(execGen.toString());
    }
}
